#!/usr/local/bin/python3

# TempoAligner.py
# Ver.0.1 (2020.06.10)
# moyashi (@hitoriblog)

import sys
import os
import subprocess

# OS標準インストール
PATH_TO_AFCONVERT = "/usr/bin/afconvert"

# Homebrewからインストール可能
PATH_TO_LAME = "/usr/local/bin/lame"
PATH_TO_RUBBERBAND = "/usr/local/bin/rubberband"
PATH_TO_NORMALIZE = "/usr/local/bin/normalize"

# ツール存在チェック
tools = [PATH_TO_LAME, PATH_TO_RUBBERBAND, PATH_TO_NORMALIZE]
for tool in tools:
    if (not os.path.exists(tool)):
        print("ERROR: " + tool + " is not exist.")
        # Homebrewでインストール
        print("Install " + os.path.basename(tool) + " from Homebrew.")
        subprocess.call(["brew", "install", os.path.basename(tool)])

# pip3からインストール可能
PATH_TO_AUBIO = "/usr/local/bin/aubio"

# ツール存在チェック
tools = [PATH_TO_AUBIO]
for tool in tools:
    if (not os.path.exists(tool)):
        print("ERROR: " + tool + " is not exist.")
        # pip3でインストール
        print("Install " + os.path.basename(tool) + " from Homebrew.")
        subprocess.call(["pip3", "install", os.path.basename(tool)])

if (len(sys.argv) == 1):
    # 引数足りない
    print("ERROR: Args not enough.")
    this_py = sys.argv[0]
    print("Example:")
    print(" " + this_py + " infile.mp3")
    print(" " + this_py + " infile.mp3 123     // actual_tempo -> 123 tempo_after_change -> default")
    print(" " + this_py + " infile.mp3 123 180 // actual_tempo -> 123 tempo_after_change -> 180")
    exit()
elif (len(sys.argv) == 2):
    # 実際のテンポ自動判定、変更後のテンポ180決め打ち
    # tempo.py hoge.mp3
    actual_tempo = None
    tempo_after_the_change = 180
elif (len(sys.argv) == 3):
    # 実際のテンポ手動指定、変更後のテンポ180決め打ち
    # tempo.py hoge.mp3 123
    actual_tempo = int(sys.argv[2])
    tempo_after_the_change = 180
elif (len(sys.argv) == 4):
    # 実際のテンポ手動指定、変更後のテンポ手動設定
    # tempo.py hoge.mp3 123 180
    actual_tempo = int(sys.argv[2])
    tempo_after_the_change = int(sys.argv[3])

# 変更後のテンポの半分のテンポ
half_tempo_after_the_change = int(tempo_after_the_change / 2)

# 処理用ファイル名生成
f = sys.argv[1]
f_wo_ext = os.path.splitext(os.path.basename(f))[0]
fw = f_wo_ext + ".wav"
fwf = f_wo_ext + "_BPM" + str(tempo_after_the_change) + ".wav"
fws = f_wo_ext + "_BPM" + str(half_tempo_after_the_change) + ".wav"

# ファイル存在チェック
if (not os.path.exists(f)):
    print("ERROR: " + f + " is not exist.")
    exit()
else:
    # カレントディレクトリ移動
    os.chdir(os.path.dirname(f))

# 入力ファイルの拡張子を取得
ext = os.path.splitext(f)[1].lower()

if (ext == ".mp3"):
    # mp3だったらLAMEでmp3からwavをデコードして生成
    subprocess.call([PATH_TO_LAME, "--decode", f, fw])
elif (ext == ".m4a"):
    # m4aだったらafconvertでm4aからwavをデコードして生成
    subprocess.call([PATH_TO_AFCONVERT, "-f", "WAVE", "-d", "LEI24", f, fw])
else:
    # 想定外のファイル種別なのでエラーで終了
    print("ERROR: " + f + " is unexpected file types.")
    exit()

# normalizeでwavをノーマライズ
subprocess.call([PATH_TO_NORMALIZE, fw])

# 実際のテンポが自動判定の場合、aubioでテンポ自動判定
if (actual_tempo == None):
    res = subprocess.check_output([PATH_TO_AUBIO, "tempo", fw])
    actual_tempo = float(res.decode().replace(" bpm\n", ""))

def calcurate_change_rate(_actual_tempo, _tempo_after_the_change):
    tmp = _tempo_after_the_change / _actual_tempo
    if (tmp >= 1):
        tmp = tmp - 1
    else:
        tmp = - (1 - tmp)
    tmp = round(tmp * 100, 2)
    if (tmp > 0):
        return "+" + str(tmp) + "%"
    else:
        return str(tmp) + "%"

rate_of_the_faster_tempo = calcurate_change_rate(actual_tempo, tempo_after_the_change)

rate_of_the_slower_tempo = calcurate_change_rate(actual_tempo, half_tempo_after_the_change)

subprocess.call([PATH_TO_RUBBERBAND, "--tempo=" + str(actual_tempo) + ":" + str(tempo_after_the_change), fw, fwf])
subprocess.call([PATH_TO_RUBBERBAND, "--tempo=" + str(actual_tempo) + ":" + str(half_tempo_after_the_change), fw, fws])

print (fwf + ": " + rate_of_the_faster_tempo)
print (fws + ": " + rate_of_the_slower_tempo)
